/*
TC-Data OSC Processing Example
 Kevin Schlei
 05.2014
 */

/*
 Setup:
 
 1. Connect the iPad and computer to the same WiFi network
 2. Input the IP address of the computer in TC-Data >> Patch >> Utilities >> OSC Setup
 3. Import the appropriate OSC libraries into your Processing patch. Here we use oscP5. 
 If you do not have oscP5 installed, see http://www.sojamo.de/libraries/oscp5/
 
 This example can be used with the default new OSC patch in TC-Data.
 */

import oscP5.*;
import netP5.*;

OscP5 oscP5;

// 4. Make sure the port number matches this value
int oscPort;

//maximum touches global variable:
int maxTouches;

//add two arrays to store the voiced data and trigger values:
float[] dataValues;
boolean[] triggerValues;

void setup() {
  size(400, 400);
  smooth();
  colorMode(HSB, 100);

  /* start oscP5, listening for incoming messages at port 9999 */
  oscPort = 9999;
  oscP5 = new OscP5(this, oscPort);

  /* maxTouches should be the same as the setting in TC-Data >> Settings >> General >> Max Touches */
  maxTouches = 11;

  //initialize the value arrays
  dataValues = new float[maxTouches];
  triggerValues = new boolean[maxTouches];
  for (int i=0; i<maxTouches; i++) {
    dataValues[i] = 0;
    triggerValues[i] = false;
  }

  //  ellipseMode(CORNERS);
}

void draw() {
  background(0);

  float ySpacer = height / float(maxTouches);

  pushMatrix();
  translate(10, 0);
  for (int i=0; i<maxTouches; i++) {
    float alpha = (triggerValues[i]) ? 255 : 30;
    fill(60, 16, 80, alpha);
    ellipse(dataValues[i], i * ySpacer + 10, 20, 20);
  }
  popMatrix();
}

void parseDataMessage(OscMessage theOscMessage) {
  float voice = theOscMessage.get(0).floatValue();
  float value = theOscMessage.get(1).floatValue();
  float newValue = value * width; //scale to screen width, or change the range in the TC-Data patch

  /* group controllers send a voice of -1. here we set all values to the group value */
  if (voice == -1) {
    for (int i=0; i<maxTouches; i++) {
      dataValues[i] = newValue;
    }
  }
  if (voice < maxTouches) {
    dataValues[int(voice)] = newValue;
  }
}

void parseTriggerMessage(OscMessage theOscMessage) {
  float voice = theOscMessage.get(0).floatValue();
  float value = theOscMessage.get(1).floatValue();
  boolean newValue = (value == 1) ? true : false;

  /* group triggerValues send a voice of -1. here we set all triggerValues to the group trigger */
  if (voice == -1) {
    for (int i=0; i<maxTouches; i++) {
      triggerValues[i] = newValue;
    }
  }
  if (voice < maxTouches) {
    triggerValues[int(voice)] = newValue;
  }
}

void oscEvent(OscMessage theOscMessage) {
  /* print the address pattern and the typetag of the received OscMessage */
  print(" addrpattern: "+theOscMessage.addrPattern());
  println(" typetag: "+theOscMessage.typetag());

  /* handle data messages */
  if (theOscMessage.checkAddrPattern("/tcdata/0-data")) {
    parseDataMessage(theOscMessage);
  }

  /* handle trigger messages */
  if (theOscMessage.checkAddrPattern("/tcdata/0-trigger") ||
    theOscMessage.checkAddrPattern("/tcdata/1-trigger")) {
    parseTriggerMessage(theOscMessage);
  }
}

